/*
 * Copyright © 2023 ESWIN.  All rights reserved.
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
 * all intellectual property and proprietary rights in and to this software.Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed,reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 * @Description: Common defination of video decode
 * @Author: ESWIN
 */

#ifndef __ES_COMM_VDEC_H__
#define __ES_COMM_VDEC_H__
#include "es_type.h"
#include "es_common.h"
#include "es_errno.h"
#include "es_comm_video.h"
#include "es_defines.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* End of #ifdef __cplusplus */

#define ES_IO_BLOCK ES_TRUE
#define ES_IO_NOBLOCK ES_FALSE

typedef enum esVIDEO_MODE_E {
    VIDEO_MODE_STREAM = 0, /* send by stream */
    VIDEO_MODE_FRAME,      /* send by frame  */
    VIDEO_MODE_COMPAT,     /* One frame supports multiple packets sending. The current frame is considered to end when
                              bEndOfFrame is equal to ES_TRUE */
    VIDEO_MODE_BUTT
} VIDEO_MODE_E;

typedef struct esVDEC_ATTR_VIDEO_S {
    ES_U32 refFrameNum; /* RW, Range: [0, 16]; reference frame num. */
} VDEC_ATTR_VIDEO_S;

typedef struct esVDEC_GRP_ATTR_S {
    PAYLOAD_TYPE_E type;  /* RW; video type to be decoded   */
    VIDEO_MODE_E mode;    /* RW; send by stream or by frame */
    ES_U32 picWidth;      /* RW; max pic width */
    ES_U32 picHeight;     /* RW; max pic height */
    ES_U32 streamBufSize; /* RW; stream buffer size(Byte) */
    ES_U32 frameBufSize;  /* RW; frame buffer size(Byte) */
    ES_U32 frameBufCnt;
    ES_U32 align;
    union {
        VDEC_ATTR_VIDEO_S vdecVideoAttr; /* structure with video ( h264/h265) */
    };
} VDEC_GRP_ATTR_S;

typedef struct esVDEC_STREAM_S {
    ES_U32 len;             /* W; stream len */
    ES_U64 PTS;             /* W; time stamp */
    ES_BOOL bEndOfFrame;    /* W; is the end of a frame */
    ES_BOOL bEndOfStream;   /* W; is the end of all stream */
    ES_BOOL bDisplay;       /* W; is the current frame displayed. only valid by VIDEO_MODE_FRAME */
    ES_U8* ATTRIBUTE pAddr; /* W; stream address */
} VDEC_STREAM_S;

typedef struct esSCALE_S {
    ES_BOOL bEnable;       /*whether scaling is enabled*/
    ES_U32 scaleWidth;     /*scaled output width*/
    ES_U32 scaleHeight;    /*scaled output height*/
    ES_U32 scale_by_ratio; /**< scaling output size will be calculated by ratio */
    ES_U32 ratio_x;        /**< 0 indicate flexiable mode, or 1/2/4/8 indicate ratio */
    ES_U32 ratio_y;        /**< 0 indicate flexiable mode, or 1/2/4/8 indicate ratio */
} SCALE_S;

typedef struct esVDEC_CHN_MODE_S {
    CROP_INFO_S cropParam;
    PIXEL_FORMAT_E pixelFormat; /* RW; output pixel format */
    ES_U32 alpha;               /* RW, Range: [0, 255]; value 0 is transparent.
                                    [0 ,127] is deemed to transparent when pixelFormat is ARGB1555 or ABGR1555
                                    [128 ,255] is deemed to non-transparent when pixelFormat is ARGB1555 or ABGR1555 */
    COLOR_GAMUT_E colorGamut;
    SCALE_S scale;
} VDEC_CHN_MODE_S;

typedef struct esVDEC_USERDATA_S {
    ES_U64 phyAddr;         /* R; userdata data phy address */
    ES_U32 len;             /* R; userdata data len */
    ES_BOOL bValid;         /* R; is valid? */
    ES_U8* ATTRIBUTE pAddr; /* R; userdata data vir address */
} VDEC_USERDATA_S;

typedef struct esVDEC_DECODE_ERROR_S {
    ES_S32 formatErr;            /* R; format error. eg: do not support filed */
    ES_S32 picSizeErrSet;        /* R; picture width or height is larger than chnnel width or height*/
    ES_S32 streamUnsprt;         /* R; unsupport the stream specification */
    ES_S32 packErr;              /* R; stream package error */
    ES_S32 prtclNumErrSet;       /* R; protocol num is not enough. eg: slice, pps, sps */
    ES_S32 picBufSizeErrSet;     /* R; the buffer size of picture is not enough */
    ES_S32 vdecStreamNotRelease; /* R; the stream not released for too long time */
    ES_S32 vdecHardwareErr;      /* R: hardware encountered an unrecoverable error*/
    ES_S32 memErrSet;            /* R; the memory is not enough */
} VDEC_DECODE_ERROR_S;

typedef struct esVDEC_GRP_STATUS_S {
    PAYLOAD_TYPE_E type;            /* R; video type to be decoded */
    ES_U32 leftStreamBytes;         /* R; left stream bytes waiting for decode */
    ES_U32 leftStreamFrames;        /* R; left frames waiting for decode,only valid for VIDEO_MODE_FRAME*/
    ES_U32 leftPics;                /* R; pics waiting for output */
    ES_U32 leftResendPics;          /* R; pics waiting for sending to bind dest */
    ES_BOOL bStartRecvStream;       /* R; had started recv stream? */
    ES_U32 recvStreamFrames;        /* R; how many frames of stream has been received. valid when send by frame. */
    ES_U32 decodeStreamFrames;      /* R; how many frames of stream has been decoded. valid when send by frame. */
    VDEC_DECODE_ERROR_S vdecDecErr; /* R; information about decode error */
} VDEC_GRP_STATUS_S;

typedef enum esVIDEO_DEC_MODE_E {
    VIDEO_DEC_MODE_IPB = 0,
    VIDEO_DEC_MODE_I,
    VIDEO_DEC_MODE_BUTT
} VIDEO_DEC_MODE_E;

typedef enum esVIDEO_OUTPUT_ORDER_E {
    VIDEO_OUTPUT_ORDER_DISP = 0,
    VIDEO_OUTPUT_ORDER_DEC,
    VIDEO_OUTPUT_ORDER_BUTT
} VIDEO_OUTPUT_ORDER_E;

typedef struct esVDEC_PARAM_VIDEO_S {
    ES_S32 errThreshold;      /* RW, Range: [0, 100]; threshold for stream error process, 0: discard with any error, 100
                                     : keep data with any error */
    VIDEO_DEC_MODE_E decMode; /* RW; decode mode , 0: deocde IPB frames, 1: only decode I frame & P frame , 2: only
                                   decode I frame */
    VIDEO_OUTPUT_ORDER_E
    outputOrder; /* RW; frames output order ,0: the same with display order , 1: the same width decoder order */
} VDEC_PARAM_VIDEO_S;

typedef struct esVDEC_GRP_PARAM_S {
    PAYLOAD_TYPE_E type;               /* RW; video type to be decoded   */
    ES_U32 displayFrameNum;            /* RW, Range: [0, 17]; display frame num */
    VDEC_PARAM_VIDEO_S vdecVideoParam; /* structure with video ( h265/h264) */
} VDEC_GRP_PARAM_S;

typedef struct esVDEC_PRTCL_PARAM_S {
    PAYLOAD_TYPE_E type; /* RW; video type to be decoded, only h264 and h265 supported*/
} VDEC_PRTCL_PARAM_S;

typedef struct esVDEC_GRP_POOL_S {
    VB_POOL picVbPool; /* RW;  vb pool id for pic buffer */
} VDEC_GRP_POOL_S;

typedef enum esVDEC_EVNT_E {
    VDEC_EVNT_STREAM_ERR = 1,
    VDEC_EVNT_UNSUPPORT,
    VDEC_EVNT_OVER_REFTHR,
    VDEC_EVNT_REF_NUM_OVER,
    VDEC_EVNT_SLICE_NUM_OVER,
    VDEC_EVNT_SPS_NUM_OVER,
    VDEC_EVNT_PPS_NUM_OVER,
    VDEC_EVNT_PICBUF_SIZE_ERR,
    VDEC_EVNT_SIZE_OVER,
    VDEC_EVNT_IMG_SIZE_CHANGE,
    VDEC_EVNT_VPS_NUM_OVER,
    VDEC_EVNT_BUTT
} VDEC_EVNT_E;

typedef struct esVDEC_MOD_PARAM_S {
    VB_SOURCE_E vdecVBSource; /* RW, Range: [1, 3];  frame buffer mode  */
    ES_U32 miniBufMode;       /* RW, Range: [0, 1];  stream buffer mode */
} VDEC_MOD_PARAM_S;

/************************************************************************************************************************/

typedef enum esVDEC_ERR_CODE_E {
    VDEC_ERR_FATAL = 20, /* unrecoverable error,such as hardware error */
    VDEC_ERR_BUTT,
} VDEC_ERR_CODE_E;

/* invlalid channel ID */
#define ES_ERR_VDEC_INVALID_CHNID ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_INVALID_CHNID)
/* at lease one parameter is illagal ,eg, an illegal enumeration value  */
#define ES_ERR_VDEC_ILLEGAL_PARAM ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_ILLEGAL_PARAM)
/* channel exists */
#define ES_ERR_VDEC_EXIST ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_EXIST)
/* using a NULL point */
#define ES_ERR_VDEC_NULL_PTR ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_NULL_PTR)
/* try to enable or initialize system,device or channel, before configing attribute */
#define ES_ERR_VDEC_NOT_CONFIG ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_CONFIG)
/* operation is not supported by NOW */
#define ES_ERR_VDEC_NOT_SUPPORT ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_SUPPORT)
/* operation is not permitted ,eg, try to change static attribute */
#define ES_ERR_VDEC_NOT_PERM ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_NOT_PERM)
/* the channle is not existed  */
#define ES_ERR_VDEC_UNEXIST ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_UNEXIST)
/* failure caused by malloc memory */
#define ES_ERR_VDEC_NOMEM ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_NOMEM)
/* failure caused by malloc buffer */
#define ES_ERR_VDEC_NOBUF ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_NOBUF)
/* no data in buffer */
#define ES_ERR_VDEC_BUF_EMPTY ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_EMPTY)
/* no buffer for new data */
#define ES_ERR_VDEC_BUF_FULL ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_BUF_FULL)
/* system is not ready,had not initialed or loaded*/
#define ES_ERR_VDEC_SYS_NOTREADY ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_SYS_NOTREADY)
/*system busy*/
#define ES_ERR_VDEC_BUSY ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_BUSY)
/* bad address,  eg. used for copy_from_user & copy_to_user*/
#define ES_ERR_VDEC_BADADDR ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, EN_ERR_BADADDR)
/* unrecoverable error,such as hardware error */
#define ES_ERR_VDEC_FATAL ES_DEF_ERR(ES_ID_VDEC, EN_ERR_LEVEL_ERROR, VDEC_ERR_FATAL)

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* End of #ifdef __cplusplus */

#endif /* End of #ifndef  __ES_COMM_VDEC_H__ */
