/*
 * Copyright © 2023 ESWIN.  All rights reserved.
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
 * all intellectual property and proprietary rights in and to this software.Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed,reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 * @Description: vdec api header file
 * @Author: ESWIN
 */

#ifndef __ES_VDEC_H__
#define __ES_VDEC_H__

#include "es_common.h"
#include "es_comm_video.h"
#include "es_comm_vdec.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* End of #ifdef __cplusplus */

/**
 * @description: Initialize decoder. This api can be called repeatedly, and success is returned.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_Init();
/**
 * @description: Deinitializing decoder. This api can be called repeatedly, and success is returned.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_Deinit();

/**
 * @description: Creates a VDEC group.
 * @param vdGrp VDEC group ID.
 * @param nId VDEC group die id. Value range[ES_DIE_0, ES_DIE_1]
 * @param pAttr Pointer to the VDEC group attributes.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_CreateGrp(VDEC_GRP vdGrp, DIE_IDX nId, const VDEC_GRP_ATTR_S *pAttr);
/**
 * @description: Creates a VDEC group.
 * @param vdGrp VDEC group ID.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_DestroyGrp(VDEC_GRP vdGrp);

/**
 * @description: Gets the attributes of a VDEC group.
 * @param vdGrp VDEC group ID.
 * @param pAttr Pointer to the VDEC group attributes.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_GetGrpAttr(VDEC_GRP vdGrp, VDEC_GRP_ATTR_S *pAttr);
/**
 * @description: Sets the attributes of a VDEC group.
 * @param vdGrp VDEC group ID.
 * @param pAttr Pointer to the VDEC group attributes.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_SetGrpAttr(VDEC_GRP vdGrp, const VDEC_GRP_ATTR_S *pAttr);

/**
 * @description: Gets the params of a VDEC channel.
 * @param vdGrp VDEC group ID.
 * @param vdChn VDEC channel ID.
 * @param pMode Pointer to the VDEC channel params.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_GetChnMode(VDEC_GRP vdGrp, VDEC_CHN vdChn, VDEC_CHN_MODE_S *pMode);
/**
 * @description: Sets the params of a VDEC channel.
 * @param vdGrp VDEC group ID.
 * @param vdChn VDEC channel ID.
 * @param pMode Pointer to the VDEC channel params.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_SetChnMode(VDEC_GRP vdGrp, VDEC_CHN vdChn, const VDEC_CHN_MODE_S *pMode);

/**
 * @description: Enable a channel of group.
 * @param vdGrp VDEC group ID.
 * @param vdChn VDEC channel ID.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_EnableChn(VDEC_GRP vdGrp, VDEC_CHN vdChn);
/**
 * @description: Disable a channel of group.
 * @param vdGrp VDEC group ID.
 * @param vdChn VDEC channel ID.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_DisableChn(VDEC_GRP vdGrp, VDEC_CHN vdChn);

/**
 * @description: Starts to receive the streams sent by the user.
 * @param vdGrp VDEC group ID.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_StartRecvStream(VDEC_GRP vdGrp);
/**
 * @description: Stops receiving the streams sent by the user.
 * @param vdGrp VDEC group ID.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_StopRecvStream(VDEC_GRP vdGrp);

/**
 * @description: Queries the status of a VDEC group.
 * @param vdGrp VDEC group ID.
 * @param pStatus Pointer to the VDEC group status.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_QueryStatus(VDEC_GRP vdGrp, VDEC_GRP_STATUS_S *pStatus);

/**
 * @description: Gets the device FD corresponding to a VDEC channel.
 * @param vdGrp VDEC group ID.
 * @param vdChn VDEC channel ID.
 * @return Negative number for error, others for success
 */
ES_S32 ES_VDEC_GetFd(VDEC_GRP vdGrp, VDEC_CHN vdChn);
/**
 * @description: Closes the device FD corresponding to a VDEC channel.
 * @param vdGrp VDEC group ID.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_CloseFd(VDEC_GRP vdGrp, VDEC_CHN vdChn);

/**
 * @description: Resets a VDEC group.
 * @param vdGrp VDEC group ID.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_ResetGrp(VDEC_GRP vdGrp);

/**
 * @description: Sets the parameters of a VDEC group.
 * @param vdGrp VDEC group ID.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_SetGrpParam(VDEC_GRP vdGrp, const VDEC_GRP_PARAM_S *pParam);
/**
 * @description: Gets the parameters of a VDEC group.
 * @param vdGrp VDEC group ID.
 * @param pMode Pointer to the VDEC group params.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_GetGrpParam(VDEC_GRP vdGrp, VDEC_GRP_PARAM_S *pParam);

/**
 * @description: Sends streams to a VDEC group.
 * @param vdGrp VDEC group ID.
 * @param pStream Pointer to the decoded stream data
 * @param milliSec -1 is block, 0 is non-block, other positive number is timeout.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_SendStream(VDEC_GRP vdGrp, const VDEC_STREAM_S *pStream, ES_S32 milliSec);

/**
 * @description: Obtains decoded images from a VDEC group.
 * @param vdGrp VDEC group ID.
 * @param vdChn VDEC channel ID.
 * @param pFrameInfo Pointer to information about the decoded images that are obtained
 * @param milliSec -1 is block, 0 is non-block, other positive number is timeout.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_GetFrame(VDEC_GRP vdGrp, VDEC_CHN vdChn, VIDEO_FRAME_INFO_S *pFrameInfo, ES_S32 milliSec);
/**
 * @description: Releases the buffer for the decoded images of a VDEC group.
 * @param vdGrp VDEC group ID.
 * @param vdChn VDEC channel ID.
 * @param pFrameInfo Pointer to information about the decoded images that are obtained by calling ES_VDEC_GetFrame
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_ReleaseFrame(VDEC_GRP vdGrp, VDEC_CHN vdChn, const VIDEO_FRAME_INFO_S *pFrameInfo);

/**
 * @description: Obtains user data in a VDEC group.
 * @param vdGrp VDEC group ID.
 * @param pUserData Pointer to the decoded user data that is obtained.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_GetUserData(VDEC_GRP vdGrp, VDEC_USERDATA_S *pUserData, ES_S32 milliSec);
/**
 * @description: Releases the buffer for the user data of a VDEC group
 * @param vdGrp VDEC group ID.
 * @param pUserData Pointer to the decoded user data that is obtained by calling HI_MPI_VDEC_GetUserData.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_ReleaseUserData(VDEC_GRP vdGrp, const VDEC_USERDATA_S *pUserData);

/**
 * @description: Sets user picture attributes.
 * @param vdGrp VDEC group ID.
 * @param vdChn VDEC channel ID.
 * @param pUsrPic Pointer to the structure of the user picture attributes
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_SetUserPic(VDEC_GRP vdGrp, VDEC_CHN vdChn, const VIDEO_FRAME_INFO_S *pUsrPic);
/**
 * @description: Enables the function of inserting a user picture.
 * @param vdGrp VDEC group ID.
 * @param vdChn VDEC channel ID.
 * @param bInstant Enabled mode of inserting a user picture. Range [0, 1]
 * 0: The mode of inserting a user picture later is enabled.
 * 1: The mode of inserting a user picture immediately is enabled.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_EnableUserPic(VDEC_GRP vdGrp, VDEC_CHN vdChn, ES_BOOL bInstant);
/**
 * @description: Disables the function of inserting a user picture.
 * @param vdGrp VDEC group ID.
 * @param vdChn VDEC channel ID.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_DisableUserPic(VDEC_GRP vdGrp, VDEC_CHN vdChn);

/**
 * @description:Sets the display mode.
 * @param vdGrp VDEC group ID.
 * @param vdChn VDEC channel ID.
 * @param displayMode Display mode enumeration
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_SetDisplayMode(VDEC_GRP vdGrp, VDEC_CHN vdChn, VIDEO_DISPLAY_MODE_E displayMode);
/**
 * @description: Gets the display mode.
 * @param vdGrp VDEC group ID.
 * @param vdChn VDEC channel ID.
 * @param displayMode Display mode enumeration
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_GetDisplayMode(VDEC_GRP vdGrp, VDEC_CHN vdChn, VIDEO_DISPLAY_MODE_E *pDisplayMode);

/**
 * @description: Binds a VDEC group to a VB pool.
 * @param vdGrp VDEC group ID.
 * @param pPool Pointer to VB pool ID
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_AttachVbPool(VDEC_GRP vdGrp, const VDEC_GRP_POOL_S *pPool);
/**
 * @description: Unbinds a VDEC group from a VB pool.
 * @param vdGrp VDEC group ID.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_DetachVbPool(VDEC_GRP vdGrp);

/**
 * @description: Sets the parameters of a VDEC group.
 * @param pModParam Pointer to the module parameter.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_SetModParam(const VDEC_MOD_PARAM_S *pModParam);
/**
 * @description: Gets the parameters of a VDEC group.
 * @param pModParam Pointer to the module parameter.
 * @return ES_SUCCESS for success, VDEC error codes for others
 */
ES_S32 ES_VDEC_GetModParam(VDEC_MOD_PARAM_S *pModParam);

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* End of #ifdef __cplusplus */

#endif /* End of #ifndef  __ES_VDEC_H__ */
