/*
 * Copyright © 2023 ESWIN.  All rights reserved.
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
 * all intellectual property and proprietary rights in and to this software.Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed,reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 * @Description: venc api header file
 * @Author: ESWIN
 */

#ifndef __ES_VENC_H__
#define __ES_VENC_H__

#include "es_common.h"
#include "es_comm_video.h"
#include "es_comm_venc.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* __cplusplus */

/**
 * @description: Initialize encoder. This api can be called repeatedly, and success is returned.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_Init(ES_VOID);
/**
 * @description: Deinitializing encoder. This api can be called repeatedly, and success is returned.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_Deinit(ES_VOID);

/**
 * @description: Creates a VENC channel.
 * @param veChn VENC channel ID.
 * @param nId VENC channel die id. Value range[ES_DIE_0, ES_DIE_1]
 * @param pAttr Pointer to the VENC channel attributes.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_CreateChn(VENC_CHN veChn, DIE_IDX nId, const VENC_CHN_ATTR_S *pAttr);
/**
 * @description: Creates a VENC channel.
 * @param veChn VENC channel ID.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_DestroyChn(VENC_CHN veChn);
/**
 * @description: Resets a VENC channel.
 * @param veChn VENC channel ID.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_ResetChn(VENC_CHN veChn);

/**
 * @description: Starts to receive the images sent by the user.
 * @param veChn VENC channel ID.
 * @param pRecvParam Pointer to the received image parameter that indicates the number of frames to be received.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_StartRecvFrame(VENC_CHN veChn, const VENC_RECV_PIC_PARAM_S *pRecvParam);
/**
 * @description: Stops receiving the images sent by the user.
 * @param veChn VENC channel ID.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_StopRecvFrame(VENC_CHN veChn);

/**
 * @description: Queries the status of a VENC channel.
 * @param veChn VENC channel ID.
 * @param pStatus Pointer to the VENC channel status.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_QueryStatus(VENC_CHN veChn, VENC_CHN_STATUS_S *pStatus);

/**
 * @description: Gets the attributes of a VENC channel.
 * @param veChn VENC channel ID.
 * @param pAttr Pointer to the VENC channel attributes.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetChnAttr(VENC_CHN veChn, const VENC_CHN_ATTR_S *pChnAttr);
/**
 * @description: Sets the attributes of a VENC channel.
 * @param veChn VENC channel ID.
 * @param pAttr Pointer to the VENC channel attributes.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetChnAttr(VENC_CHN veChn, VENC_CHN_ATTR_S *pChnAttr);

/**
 * @description: Obtains encoded streams from a VENC channel.
 * @param veChn VENC channel ID.
 * @param pStream Pointer to the stream structure
 * @param milliSec -1 is block, 0 is non-block, other positive number is timeout.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetStream(VENC_CHN veChn, VENC_STREAM_S *pStream, ES_S32 milliSec);
/**
 * @description: Releases the stream buf for the encoded stream of a VENC channel.
 * @param veChn VENC channel ID.
 * @param pStream Pointer to stream structure that are obtained by calling ES_VENC_GetFrame
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_ReleaseStream(VENC_CHN veChn, VENC_STREAM_S *pStream);

/**
 * @description: Inserts user data.
 * @param veChn VENC channel ID.
 * @param pData Pointer to user data
 * @param len User data length (unit: byte)
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_InsertUserData(VENC_CHN veChn, ES_U8 *pData, ES_U32 len);

/**
 * @description: Sends raw images for encoding.
 * @param veChn VENC channel ID.
 * @param pFrame Pointer to a raw image structure
 * @param milliSec -1 is block, 0 is non-block, other positive number is timeout.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SendFrame(VENC_CHN veChn, const VIDEO_FRAME_INFO_S *pFrame, ES_S32 milliSec);
/**
 * @description: Sends raw images with QpMap table for encoding.
 * @param veChn VENC channel ID.
 * @param pFrame Pointer to a raw image structure with RC information.
 * @param milliSec -1 is block, 0 is non-block, other positive number is timeout.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SendFrameEx(VENC_CHN veChn, const USER_FRAME_INFO_S *pFrame, ES_S32 milliSec);
/**
 * @description: Requests IDR frames.
 * @param veChn VENC channel ID.
 * @param bInstant IDR frame instant encoding enable
 * 0: the IDR frame is encoded at the current frame rate after the current frame is encoded.
 * 1: the IDR frame is encoded immediately regardless of the frame rate.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_RequestIDR(VENC_CHN veChn, ES_BOOL bInstant);

/**
 * @description: Gets the device FD corresponding to a VENC channel.
 * @param veChn VENC channel ID.
 * @return Negative number for error, others for success
 */
ES_S32 ES_VENC_GetFd(VENC_CHN veChn);
/**
 * @description: Closes the device FD corresponding to a VENC channel.
 * @param veChn VENC channel ID.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_CloseFd(VENC_CHN veChn);

/**
 * @description: Sets the ROI attributes of a H.264/H.265 channel.
 * @param veChn VENC channel ID.
 * @param pROIAttr Pointer to ROI attributes of the H.264/H.265 streams.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetROIAttr(VENC_CHN veChn, const VENC_ROI_ATTR_S *pROIAttr);
/**
 * @description: Gets the ROI attributes of a H.264/H.265 channel.
 * @param veChn VENC channel ID.
 * @param pROIAttr Pointer to ROI attributes of the H.264/H.265 streams.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetROIAttr(VENC_CHN veChn, ES_U32 index, VENC_ROI_ATTR_S *pROIAttr);

/**
 * @description: Sets the ROI attributes of a JPEG/MJPEG channel.
 * @param veChn VENC channel ID.
 * @param pJPEGROIAttr Pointer to ROI attributes of the JPEG/MJPEG streams.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetJPEGROIAttr(VENC_CHN veChn, const VENC_JPEG_ROI_ATTR_S *pJPEGROIAttr);
/**
 * @description: Gets the ROI attributes of a JPEG/MJPEG channel.
 * @param veChn VENC channel ID.
 * @param pJPEGROIAttr Pointer to ROI attributes of the JPEG/MJPEG streams.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetJPEGROIAttr(VENC_CHN veChn, VENC_JPEG_ROI_ATTR_S *pJPEGROIAttr);

/**
 * @description: Sets the slice attributes of an H.264 channel.
 * @param veChn VENC channel ID.
 * @param pSliceSplit Pointer to slice mode of an H.264 stream.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetH264SliceSplit(VENC_CHN veChn, const VENC_H264_SLICE_SPLIT_S *pSliceSplit);
/**
 * @description: Gets the slice attributes of an H.264 channel.
 * @param veChn VENC channel ID.
 * @param pSliceSplit Pointer to slice mode of an H.264 stream.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetH264SliceSplit(VENC_CHN veChn, VENC_H264_SLICE_SPLIT_S *pSliceSplit);

/* Not support */
ES_S32 ES_VENC_SetH264Trans(VENC_CHN veChn, const VENC_H264_TRANS_S *pH264Trans);
ES_S32 ES_VENC_GetH264Trans(VENC_CHN veChn, VENC_H264_TRANS_S *pH264Trans);

/**
 * @description: Sets the entropy encoding mode for an H.264 channel.
 * @param veChn VENC channel ID.
 * @param pH264EntropyEnc Pointer to entropy encoding mode for an H.264 channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetH264Entropy(VENC_CHN veChn, const VENC_H264_ENTROPY_S *pH264EntropyEnc);
/**
 * @description: Gets the entropy encoding mode for an H.264 channel.
 * @param veChn VENC channel ID.
 * @param pH264EntropyEnc Pointer to entropy encoding mode for an H.264 channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetH264Entropy(VENC_CHN veChn, VENC_H264_ENTROPY_S *pH264EntropyEnc);

/**
 * @description: Sets the de-blocking attributes of an H.264 channel.
 * @param veChn VENC channel ID.
 * @param pH264Dblk Pointer to de-blocking attributes of an H.264 parameter
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetH264Dblk(VENC_CHN veChn, const VENC_H264_DBLK_S *pH264Dblk);
/**
 * @description: Gets the de-blocking attributes of an H.264 channel.
 * @param veChn VENC channel ID.
 * @param pH264Dblk Pointer to de-blocking attributes of an H.264 parameter
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetH264Dblk(VENC_CHN veChn, VENC_H264_DBLK_S *pH264Dblk);

/**
 * @description: Sets the VUI parameters of an H.264 channel.
 * @param veChn VENC channel ID.
 * @param pH264VUI Pointer to VUI structure of an H.264 channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetH264VUI(VENC_CHN veChn, const VENC_H264_VUI_S *pH264VUI);
/**
 * @description: Gets the VUI parameters of an H.264 channel.
 * @param veChn VENC channel ID.
 * @param pH264VUI Pointer to VUI structure of an H.264 channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetH264VUI(VENC_CHN veChn, VENC_H264_VUI_S *pH264VUI);

/**
 * @description: Sets the VUI parameters of an H.H265 channel.
 * @param veChn VENC channel ID.
 * @param pH265VUI Pointer to VUI structure of an H.H265 channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetH265VUI(VENC_CHN veChn, const VENC_H265_VUI_S *pH265VUI);
/**
 * @description: Gets the VUI parameters of an H.H265 channel.
 * @param veChn VENC channel ID.
 * @param pH265VUI Pointer to VUI structure of an H.H265 channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetH265VUI(VENC_CHN veChn, VENC_H265_VUI_S *pH265VUI);

/**
 * @description: Sets the advanced parameters of a JPEG channel.
 * @param veChn VENC channel ID.
 * @param pJpegParam Pointer to advanced parameters of a JPEG channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetJpegParam(VENC_CHN veChn, const VENC_JPEG_PARAM_S *pJpegParam);
/**
 * @description: Gets the advanced parameters of a JPEG channel.
 * @param veChn VENC channel ID.
 * @param pJpegParam Pointer to advanced parameters of a JPEG channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetJpegParam(VENC_CHN veChn, VENC_JPEG_PARAM_S *pJpegParam);

/**
 * @description: Sets the advanced parameters of a MJPEG channel.
 * @param veChn VENC channel ID.
 * @param pJpegParam Pointer to advanced parameters of a MJPEG channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetMjpegParam(VENC_CHN veChn, const VENC_MJPEG_PARAM_S *pMjpegParam);
/**
 * @description: Gets the advanced parameters of a MJPEG channel.
 * @param veChn VENC channel ID.
 * @param pJpegParam Pointer to advanced parameters of a MJPEG channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetMjpegParam(VENC_CHN veChn, VENC_MJPEG_PARAM_S *pMjpegParam);

/**
 * @description: Gets the advanced RC parameters of a VENC channel.
 * @param veChn VENC channel ID.
 * @param pRcParam Pointer to the advanced RC parameters of a VENC channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetRcParam(VENC_CHN veChn, VENC_RC_PARAM_S *pRcParam);
/**
 * @description: Sets the advanced RC parameters of a VENC channel.
 * @param veChn VENC channel ID.
 * @param pRcParam Pointer to the advanced RC parameters of a VENC channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetRcParam(VENC_CHN veChn, const VENC_RC_PARAM_S *pRcParam);

/* Not support */
ES_S32 ES_VENC_SetRefParam(VENC_CHN veChn, const VENC_REF_PARAM_S *pRefParam);
ES_S32 ES_VENC_GetRefParam(VENC_CHN veChn, VENC_REF_PARAM_S *pRefParam);

/**
 * @description: Sets the slice attributes of an H.265 channel.
 * @param veChn VENC channel ID.
 * @param pSliceSplit Pointer to slice mode of an H.265 stream.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetH265SliceSplit(VENC_CHN veChn, const VENC_H265_SLICE_SPLIT_S *pSliceSplit);
/**
 * @description: Sets the slice attributes of an H.265 channel.
 * @param veChn VENC channel ID.
 * @param pSliceSplit Pointer to slice mode of an H.265 stream.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetH265SliceSplit(VENC_CHN veChn, VENC_H265_SLICE_SPLIT_S *pSliceSplit);

/* Not support */
ES_S32 ES_VENC_SetH265PredUnit(VENC_CHN veChn, const VENC_H265_PU_S *pPredUnit);
ES_S32 ES_VENC_GetH265PredUnit(VENC_CHN veChn, VENC_H265_PU_S *pPredUnit);

/* Not support */
ES_S32 ES_VENC_SetH265Trans(VENC_CHN veChn, const VENC_H265_TRANS_S *pH265Trans);
ES_S32 ES_VENC_GetH265Trans(VENC_CHN veChn, VENC_H265_TRANS_S *pH265Trans);

/**
 * @description: Sets the de-blocking attributes of an H.265 channel.
 * @param veChn VENC channel ID.
 * @param pH265Dblk Pointer to de-blocking attributes of an H.265 parameter
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetH265Dblk(VENC_CHN veChn, const VENC_H265_DBLK_S *pH265Dblk);
/**
 * @description: Gets the de-blocking attributes of an H.265 channel.
 * @param veChn VENC channel ID.
 * @param pH265Dblk Pointer to de-blocking attributes of an H.265 parameter
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetH265Dblk(VENC_CHN veChn, VENC_H265_DBLK_S *pH265Dblk);

/**
 * Reserved
 * @description: Sets the SAO attributes for H.265 encoding.
 * @param veChn VENC channel ID.
 * @param pH265SAO Pointer to SAO configuration of an H.265 VENC channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetH265SAO(VENC_CHN veChn, const VENC_H265_SAO_S *pH265SAO);
/**
 * Reserved
 * @description: Gets the SAO attributes for H.265 encoding.
 * @param veChn VENC channel ID.
 * @param pH265SAO Pointer to SAO configuration of an H.265 VENC channel
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetH265SAO(VENC_CHN veChn, VENC_H265_SAO_S *pH265SAO);

/**
 * Reserved
 * @description: Gets the SSE attributes of an H.264/H.265 channel.
 * @param veChn VENC channel ID.
 * @param index Index of the SSE region for an H.264/H.265 VENC schannel
 * @param pstSSECfg Pointer to SSE parameter setting
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetSSERegion(VENC_CHN veChn, ES_U32 index, VENC_SSE_CFG_S *pSSECfg);
/**
 * Reserved
 * @description: Sets the SSE attributes of an H.264/H.265 channel.
 * @param veChn VENC channel ID.
 * @param pstSSECfg Pointer to SSE parameter setting
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetSSERegion(VENC_CHN veChn, const VENC_SSE_CFG_S *pSSECfg);

/**
 * @description: Sets the parameters of the VENC channel.
 * @param veChn VENC channel ID.
 * @param pChnParam Pointer to the VENC channel parameters
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetChnParam(VENC_CHN veChn, const VENC_CHN_PARAM_S *pChnParam);
/**
 * @description: Gets the parameters of the VENC channel.
 * @param veChn VENC channel ID.
 * @param pChnParam Pointer to the VENC channel parameters
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetChnParam(VENC_CHN veChn, VENC_CHN_PARAM_S *pChnParam);

/**
 * @description: Sets the module parameters related to encoding
 * @param pModParam Pointer to the parameters of the encoding module
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_SetModParam(const VENC_PARAM_MOD_S *pModParam);
/**
 * @description: Gets the module parameters related to encoding
 * @param veChn VENC channel ID.
 * @param pModParam Pointer to the parameters of the encoding module
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_GetModParam(VENC_PARAM_MOD_S *pModParam);

/**
 * @description: Binds a VENC channel to a VB pool.
 * @param veChn VENC channel ID.
 * @param pPool Pointer to VB pool structure.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_AttachVbPool(VENC_CHN veChn, const VENC_CHN_POOL_S *pPool);
/**
 * @description: Unbinds a VENC channel from a VB pool.
 * @param veChn VENC channel ID.
 * @return ES_SUCCESS for success, VENC error codes for others
 */
ES_S32 ES_VENC_DetachVbPool(VENC_CHN veChn);

/* Not support */
ES_S32 ES_VENC_SetFrameLostStrategy(VENC_CHN veChn, const VENC_FRAMELOST_S *pFrmLostParam);
ES_S32 ES_VENC_GetFrameLostStrategy(VENC_CHN veChn, VENC_FRAMELOST_S *pFrmLostParam);

/* Not support */
ES_S32 ES_VENC_SetSuperFrameStrategy(VENC_CHN veChn, const VENC_SUPERFRAME_CFG_S *pSuperFrmParam);
ES_S32 ES_VENC_GetSuperFrameStrategy(VENC_CHN veChn, VENC_SUPERFRAME_CFG_S *pSuperFrmParam);

/* Not support */
ES_S32 ES_VENC_SetIntraRefresh(VENC_CHN veChn, const VENC_INTRA_REFRESH_S *pIntraRefresh);
ES_S32 ES_VENC_GetIntraRefresh(VENC_CHN veChn, VENC_INTRA_REFRESH_S *pIntraRefresh);

/* Not support */
ES_S32 ES_VENC_SetCuPrediction(VENC_CHN veChn, const VENC_CU_PREDICTION_S *pCuPrediction);
ES_S32 ES_VENC_GetCuPrediction(VENC_CHN veChn, VENC_CU_PREDICTION_S *pCuPrediction);

/* Not support */
ES_S32 ES_VENC_SetSkipBias(VENC_CHN veChn, const VENC_SKIP_BIAS_S *pSkipBias);
ES_S32 ES_VENC_GetSkipBias(VENC_CHN veChn, VENC_SKIP_BIAS_S *pSkipBias);

/* Not support */
ES_S32 ES_VENC_SetRcAdvParam(VENC_CHN veChn, const VENC_RC_ADVPARAM_S *pRcAdvParam);
ES_S32 ES_VENC_GetRcAdvParam(VENC_CHN veChn, VENC_RC_ADVPARAM_S *pRcAdvParam);

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif /* __ES_VENC_H__ */
