/*
 * Copyright © 2023 ESWIN. All rights reserved.
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
 * all intellectual property and proprietary rights in and to this software.Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed,reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 * @Description: Define the API of VO
 * @Author: ESWIN
 */

#ifndef __ES_VO_H__
#define __ES_VO_H__

#include "es_comm_vo.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* __cplusplus */

/* Device Relative Settings */

/**
 * @description: The function of this interface is to initialize VO.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_Init(ES_VOID);

/**
 * @description: The purpose of this interface is to de-initialize VO.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_DeInit(ES_VOID);

/**
 * @description: This interface is used to configure the public properties of the video output device.
 * @param {VO_DEV} voDev VO device ID. Value range[0, ES_VO_MAX_DEV_NUM - 1].
 * @param {VO_PUB_ATTR_S} *pPubAttr Video output device public attribute structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetPubAttr(VO_DEV voDev, const VO_PUB_ATTR_S *pPubAttr);

/**
 * @description: The function of this interface is to obtain the public properties of the video output device.
 * @param {VO_DEV} voDev VO device ID. Value range[0, ES_VO_MAX_DEV_NUM - 1].
 * @param {VO_PUB_ATTR_S} *pPubAttr Video output device public attribute structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetPubAttr(VO_DEV voDev, VO_PUB_ATTR_S *pPubAttr);

/**
 * @description: This interface is used to set the device frame rate under the device user timing and can only be used
 *               on virtual devices.
 * @param {VO_DEV} voDev VO device ID. Value range[0, ES_VO_MAX_DEV_NUM - 1].
 * @param {ES_U32} frameRate Device frame rate. Value range(0, 240].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetDevFrameRate(VO_DEV voDev, ES_U32 frameRate);

/**
 * @description: This interface is used to obtain the frame rate of the device and can only be used on virtual devices.
 * @param {VO_DEV} voDev VO device ID. Value range[0, ES_VO_MAX_DEV_NUM - 1].
 * @param {ES_U32} *pFrameRate A pointer of type ES_U32.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetDevFrameRate(VO_DEV voDev, ES_U32 *pFrameRate);

/**
 * @description: This interface is used to enable the video output device.
 * @param {VO_DEV} voDev VO device ID. Value range[0, ES_VO_MAX_DEV_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_Enable(VO_DEV voDev, DIE_IDX nId);

/**
 * @description: This interface is used to disable video output devices.
 * @param {VO_DEV} voDev VO device ID. Value range[0, ES_VO_MAX_DEV_NUM - 1].
 * @param {DIE_IDX} nId DIE ID. Value range[0, ES_VO_MAX_PHY_DEV_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_Disable(VO_DEV voDev);

/* Video Relative Settings */

/**
 * @description: This interface is used to set the properties of the video layer.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_VIDEO_LAYER_ATTR_S} *pLayerAttr Video layer attribute structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetVideoLayerAttr(VO_LAYER voLayer, const VO_VIDEO_LAYER_ATTR_S *pLayerAttr);

/**
 * @description: The purpose of this interface is to obtain video layer properties.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_VIDEO_LAYER_ATTR_S} *pLayerAttr Video layer attribute structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetVideoLayerAttr(VO_LAYER voLayer, VO_VIDEO_LAYER_ATTR_S *pLayerAttr);

/**
 * @description: This interface enables the video layer.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_EnableVideoLayer(VO_LAYER voLayer);

/**
 * @description: This interface is used to disable the video layer.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_DisableVideoLayer(VO_LAYER voLayer);

/**
 * @description: The function of this interface is to set the video layer output image effect.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CSC_S} *pVideoCSC Image output effect structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetVideoLayerCSC(VO_LAYER voLayer, const VO_CSC_S *pVideoCSC);

/**
 * @description: The function of this interface is to obtain the output image effect of the device.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CSC_S} *pVideoCSC Image output effect structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetVideoLayerCSC(VO_LAYER voLayer, VO_CSC_S *pVideoCSC);

/**
 * @description: The role of this interface is to set the setting properties of the channel on the video layer starting.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_BatchBegin(VO_LAYER voLayer);

/**
 * @description: The role of this interface is to set the video layer on the channel Settings properties end.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_BatchEnd(VO_LAYER voLayer);

/**
 * @description: This interface is used to set the CROP feature property of the video layer.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {CROP_INFO_S} *pCropInfo Video layer CROP attribute structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetVideoLayerCrop(VO_LAYER voLayer, const CROP_INFO_S *pCropInfo);

/**
 * @description: The function of this interface is to get the CROP function property of the video layer.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {CROP_INFO_S} *pCropInfo Video layer CROP attribute structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetVideoLayerCrop(VO_LAYER voLayer, CROP_INFO_S *pCropInfo);

/**
 * @description: The function of this interface is to set the video layer amplitude ratio function.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_LAYER_PARAM_S} *pLayerParam Video layer amplitude ratio parameter pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetLayerAutoAspectRatio(VO_LAYER voLayer, const VO_LAYER_PARAM_S *pLayerParam);

/**
 * @description: The function of this interface is to obtain the video layer amplitude ratio function parameters.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_LAYER_PARAM_S} *pLayerParam ideo layer amplitude ratio parameter pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetLayerAutoAspectRatio(VO_LAYER voLayer, VO_LAYER_PARAM_S *pLayerParam);

/* Display relative operations */

/**
 * @description: This interface is used to set the playback tolerance.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {ES_U32} toleration Playback tolerance. Value range[ES_VO_MIN_PLAY_TOLERATION, ES_VO_MAX_PLAY_TOLERATION].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetPlayToleration(VO_LAYER voLayer, ES_U32 toleration);

/**
 * @description: The function of this interface is to obtain the playback tolerance.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {ES_U32} *pToleration A pointer of type ES_U32.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetPlayToleration(VO_LAYER voLayer, ES_U32 *pToleration);

/**
 * @description: The purpose of this interface is to get the video layer Fd.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {ES_S32} *pFd A pointer of type ES_S32.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetLayerFd(VO_LAYER voLayer, ES_S32 *pFd);

/**
 * @description: The function of this interface is to obtain the output screen image data.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VIDEO_FRAME_INFO_S} *pVFrame Gets pointer to output screen image data information structure.
 * @param {ES_S32} milliSec
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetScreenFrame(VO_LAYER voLayer, VIDEO_FRAME_INFO_S *pVFrame, ES_S32 milliSec);

/**
 * @description: The function of this interface is to release the output screen image data.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VIDEO_FRAME_INFO_S} *pVFrame Released pointer to output screen image data information structure.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_ReleaseScreenFrame(VO_LAYER voLayer, const VIDEO_FRAME_INFO_S *pVFrame);

/**
 * @description: The purpose of this interface is to set the length of the display buffer.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {ES_U32} bufLen Displays the length of the buffer. Value range[0] and [3].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetDisplayBufLen(VO_LAYER voLayer, ES_U32 bufLen);

/**
 * @description: The purpose of this interface is to get the length of the display buffer.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {ES_U32} *pBufLen A pointer of type ES_U32.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetDisplayBufLen(VO_LAYER voLayer, ES_U32 *pBufLen);

/* Channel Relative Operations */

/**
 * @description: This interface configures the properties of the specified video output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {VO_CHN_ATTR_S} *pChnAttr Video channel property pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetChnAttr(VO_LAYER voLayer, VO_CHN voChn, const VO_CHN_ATTR_S *pChnAttr);

/**
 * @description: The purpose of this interface is to obtain the properties of the specified video output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {VO_CHN_ATTR_S} *pChnAttr Video channel property pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetChnAttr(VO_LAYER voLayer, VO_CHN voChn, VO_CHN_ATTR_S *pChnAttr);

/**
 * @description: This interface enables the specified video output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_EnableChn(VO_LAYER voLayer, VO_CHN voChn);

/**
 * @description: This interface is used to disable the specified video output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_DisableChn(VO_LAYER voLayer, VO_CHN voChn);

/**
 * @description: The function of this interface is to configure the parameters of the specified video output channel for
 *               the amplitude ratio function.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {VO_CHN_PARAM_S} *pChnParam Video channel parameter pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetChnParam(VO_LAYER voLayer, VO_CHN voChn, const VO_CHN_PARAM_S *pChnParam);

/**
 * @description: The purpose of this interface is to get the parameters of the specified video output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {VO_CHN_PARAM_S} *pChnParam ideo channel parameter pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetChnParam(VO_LAYER voLayer, VO_CHN voChn, VO_CHN_PARAM_S *pChnParam);

/**
 * @description: This interface is used to set the display frame rate of the specified video output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {ES_S32} chnFrmRate The video channel displays the frame rate.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetChnFrameRate(VO_LAYER voLayer, VO_CHN voChn, ES_S32 chnFrmRate);

/**
 * @description: This interface is to obtain the display frame rate of the specified video output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {ES_S32} *pChnFrmRate The video channel displays the frame rate.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetChnFrameRate(VO_LAYER voLayer, VO_CHN voChn, ES_S32 *pChnFrmRate);

/**
 * @description: This interface is used to pause the specified video output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_PauseChn(VO_LAYER voLayer, VO_CHN voChn);

/**
 * @description: This interface is used to restore the specified video output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_ResumeChn(VO_LAYER voLayer, VO_CHN voChn);

/**
 * @description: The function of this interface is to play the specified video output channel in a single frame.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_StepChn(VO_LAYER voLayer, VO_CHN voChn);

/**
 * @description:This interface displays the specified channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_ShowChn(VO_LAYER voLayer, VO_CHN voChn);

/**
 * @description: This interface is used to hide the specified channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_HideChn(VO_LAYER voLayer, VO_CHN voChn);

/**
 * @description: The function of this interface is to set the video output local magnification window.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {VO_ZOOM_ATTR_S} *pZoomAttr Locally magnify the attribute structure.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetZoomInWindow(VO_LAYER voLayer, VO_CHN voChn, const VO_ZOOM_ATTR_S *pZoomAttr);

/**
 * @description: The function of this interface is to obtain the video output local magnification window.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {VO_ZOOM_ATTR_S} *pZoomAttr Locally enlarges the attribute structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetZoomInWindow(VO_LAYER voLayer, VO_CHN voChn, VO_ZOOM_ATTR_S *pZoomAttr);

/**
 * @description: The purpose of this interface is to obtain a timestamp of the image being displayed on the specified
 *               video output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {ES_U64} *pChnPTS Channel timestamp pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetChnPTS(VO_LAYER voLayer, VO_CHN voChn, ES_U64 *pChnPTS);

/**
 * @description: This interface is used to query the status of the video output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {VO_QUERY_STATUS_S} *pStatus Channel state structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_QueryChnStatus(VO_LAYER voLayer, VO_CHN voChn, VO_QUERY_STATUS_S *pStatus);

/**
 * @description: The function of the interface is to send the video image to the specified output channel for display.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {VIDEO_FRAME_INFO_S} *pVFrame Video data information pointer.
 * @param {ES_S32} milliSec Timeout period. Unit: ms.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SendFrame(VO_LAYER voLayer, VO_CHN voChn, VIDEO_FRAME_INFO_S *pVFrame, ES_S32 milliSec);

/**
 * @description: This interface is used to clear buffer data in the specified output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {ES_BOOL} bClrAll Whether to clear all data in the channel buffer. Value range:
 *        ES_TRUE: Empty all the data in the channel buffer. At this time, the area of the channel on the screen will
 *        not be displayed until a new image data arrives.
 *        ES_FALSE: Keep one image in the channel buffer and empty the other data.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_ClearChnBuf(VO_LAYER voLayer, VO_CHN voChn, ES_BOOL bClrAll);

/**
 * @description: The purpose of this interface is to set the border properties of the channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {VO_BORDER_S} *pBorder Video channel border property pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetChnBorder(VO_LAYER voLayer, VO_CHN voChn, const VO_BORDER_S *pBorder);

/**
 * @description: The purpose of this interface is to get the border properties of the channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {VO_BORDER_S} *pBorder Video channel border property pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetChnBorder(VO_LAYER voLayer, VO_CHN voChn, VO_BORDER_S *pBorder);

/**
 * @description: The role of this interface is set to specify the rotation Angle of the video output channel. Set the
 *               rotation Angle for the channel. The rotation Angle will act on the video image entering the channel.
 *               The specific effect is that the image will be displayed after the corresponding Angle is rotated.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {ROTATION_E} rotation Rotation Angle enumerated value. Value range{ROTATION_0 , ROTATION_90,
 *               ROTATION_180, ROTATION_270}。
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetChnRotation(VO_LAYER voLayer, VO_CHN voChn, ROTATION_E rotation);

/**
 * @description: The purpose of this interface is to obtain the rotation Angle of the specified video output channel.
 * @param {VO_LAYER} voLayer VO layer ID. Value range[0, ES_VO_MAX_LAYER_NUM - 1].
 * @param {VO_CHN} voChn VO channel ID. Value range[0, ES_VO_MAX_CHN_NUM - 1].
 * @param {ROTATION_E} *pRotation Rotation Angle enumeration variable pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetChnRotation(VO_LAYER voLayer, VO_CHN voChn, ROTATION_E *pRotation);

/* WBC(Write Back Control) Relative Settings */

/**
 * @description: This interface is used to enable the write back device.
 * @param {VO_WBC} voWBC VO WBC ID. Value range[0, VO_MAX_WBC_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_EnableWBC(VO_WBC voWBC);

/**
 * @description: This interface is used to disable the write back device.
 * @param {VO_WBC} voWBC VO WBC ID. Value range[0, VO_MAX_WBC_NUM - 1].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_DisableWBC(VO_WBC voWBC);

/**
 * @description: This interface is used to set the write back depth of the write back device.
 * @param {VO_WBC} voWBC VO WBC ID. Value range[0, VO_MAX_WBC_NUM - 1].
 * @param {ES_U32} depth Write back The device writes back depth. Value range[0, 8].
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_SetWBCDepth(VO_WBC voWBC, ES_U32 depth);

/**
 * @description: This interface is used to obtain the write back depth of the write back device.
 * @param {VO_WBC} voWBC VO WBC ID. Value range[0, VO_MAX_WBC_NUM - 1].
 * @param {ES_U32} *pDepth Write back The device writes back depth.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetWBCDepth(VO_WBC voWBC, ES_U32 *pDepth);

/**
 * @description: This interface is used to obtain WBC Fd.
 * @param {VO_WBC} voWBC VO WBC ID. Value range[0, VO_MAX_WBC_NUM - 1].
 * @param {ES_S32} *pFd A pointer of type ES_S32.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetWBCFd(VO_WBC voWBC, ES_S32 *pFd);

/**
 * @description: The function of this interface is to obtain the output image data of the write back device.
 * @param {VO_WBC} voWBC VO WBC ID. Value range[0, VO_MAX_WBC_NUM - 1].
 * @param {VIDEO_FRAME_INFO_S} *pVFrame Retrievals a pointer to the output image data information structure.
 * @param {ES_S32} milliSec Timeout period. Unit: ms.
 *                 Timeout parameter s32ec - When the value is -1, the interface is blocked, 0 is a non-blocking
 *                 interface, and when the value is greater than 0, the timeout waiting period is specified.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_GetWBCFrame(VO_WBC voWBC, VIDEO_FRAME_INFO_S *pVFrame, ES_S32 milliSec);

/**
 * @description: The function of this interface is to release the output image data of the write back device.
 * @param {VO_WBC} voWBC VO WBC ID. Value range[0, VO_MAX_WBC_NUM - 1].
 * @param {VIDEO_FRAME_INFO_S} *pVFrame The output channel image data information structure pointer is released.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_VO_ReleaseWBCFrame(VO_WBC voWBC, const VIDEO_FRAME_INFO_S *pVFrame);

/* HDMI Relative Operations */

/**
 * @description: This interface is used to initialize HDMI.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_HDMI_Init();

/**
 * @description: The purpose of this interface is to de-initialize the HDMI.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_HDMI_DeInit();

/**
 * @description: This interface enables HDMI output.
 * @param {ES_HDMI_ID_E} hdmiId VO Hdmi ID. Value range{ ES_HDMI_ID_0 = 0, ES_HDMI_ID_BUTT }.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_HDMI_EnableVideo(ES_HDMI_ID_E hdmiId);

/**
 * @description: This interface is used to disable HDMI output.
 * @param {ES_HDMI_ID_E} hdmiId VO Hdmi ID. Value range{ ES_HDMI_ID_0 = 0, ES_HDMI_ID_BUTT }.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_HDMI_DisableVideo(ES_HDMI_ID_E hdmiId);

/**
 * @description: This interface is used to obtain the display capability set of the HDMI Sink terminal.
 * @param {ES_HDMI_ID_E} hdmiId VO Hdmi ID. Value range{ ES_HDMI_ID_0 = 0, ES_HDMI_ID_BUTT }.
 * @param {ES_HDMI_DISPLAY_MODE_S} *pDispMode The capability set structure pointer is displayed on the HDMI Sink
 *               terminal.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_HDMI_GetDispMode(ES_HDMI_ID_E hdmiId, ES_HDMI_DISPLAY_MODE_S *pDispMode);

/**
 * @description: This interface is used to set HDMI properties.
 * @param {ES_HDMI_ID_E} hdmiId VO Hdmi ID. Value range{ ES_HDMI_ID_0 = 0, ES_HDMI_ID_BUTT }.
 * @param {ES_HDMI_ATTR_S} *pAttr HDMI attribute structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_HDMI_SetAttr(ES_HDMI_ID_E hdmiId, const ES_HDMI_ATTR_S *pAttr);

/**
 * @description: This interface is used to obtain HDMI properties.
 * @param {ES_HDMI_ID_E} hdmiId VO Hdmi ID. Value range{ ES_HDMI_ID_0 = 0, ES_HDMI_ID_BUTT }.
 * @param {ES_HDMI_ATTR_S} *pAttr HDMI attribute structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_HDMI_GetAttr(ES_HDMI_ID_E hdmiId, ES_HDMI_ATTR_S *pAttr);

/**
 * @description: This interface is used to register HotPlug event callbacks.
 * @param {ES_HDMI_ID_E} hdmiId VO Hdmi ID. Value range{ ES_HDMI_ID_0 = 0, ES_HDMI_ID_BUTT }.
 * @param {ES_HDMI_CALLBACK_FUNC_S} *pCallbackFunc Pointer to HotPlug event callback function.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_HDMI_RegCallbackFunc(ES_HDMI_ID_E hdmiId, ES_HDMI_CALLBACK_FUNC_S *pCallbackFunc);

/**
 * @description: This interface is used to cancel the HotPlug event callback.
 * @param {ES_HDMI_ID_E} hdmiId VO Hdmi ID. Value range{ ES_HDMI_ID_0 = 0, ES_HDMI_ID_BUTT }.
 * @param {ES_HDMI_CALLBACK_FUNC_S} *pCallbackFunc Pointer to HotPlug event callback function.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_HDMI_UnRegCallbackFunc(ES_HDMI_ID_E hdmiId, ES_HDMI_CALLBACK_FUNC_S *pCallbackFunc);

/**
 * @description: This interface is used to cancel the HotPlug event callback.
 * @param {ES_HDMI_ID_E} hdmiId VO Hdmi ID. Value range{ ES_HDMI_ID_0 = 0, ES_HDMI_ID_BUTT }.
 * @param {ES_HDMI_EDID_S} *pEdidData HDMI EDID information structure.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_HDMI_Force_GetEDID(ES_HDMI_ID_E hdmiId, ES_HDMI_EDID_S *pEdidData);

/* MIPI Relative Operations */

/**
 * @description: This interface is used to initialize MIPI.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_MIPI_Init();

/**
 * @description: This interface is used to de-initialize MIPI.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_MIPI_DeInit();

/**
 * @description: This interface is used to set mipi device properties.
 * @param {ES_MIPI_ID_E} mipiDev MIPI Device ID. Value range{ ES_MIPI_ID_0 = 0, ES_MIPI_ID_BUTT }.
 * @param {MIPI_DEV_ATTR_S} *pDevAttr HDMI device property structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_MIPI_SetDevAttr(ES_MIPI_ID_E mipiDev, const MIPI_DEV_ATTR_S *pDevAttr);

/**
 * @description: This interface is used to set initialization commands.
 * @param {ES_MIPI_ID_E} mipiDev MIPI Device ID. Value range{ ES_MIPI_ID_0 = 0, ES_MIPI_ID_BUTT }.
 * @param {ES_MIPI_CMD_S} *pMipiCmd HDMI command structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_MIPI_SetInitCmd(ES_MIPI_ID_E mipiDev, const ES_MIPI_CMD_S *pMipiCmd);

/**
 * @description: This interface is used to set the enable command.
 * @param {ES_MIPI_ID_E} mipiDev MIPI Device ID. Value range{ ES_MIPI_ID_0 = 0, ES_MIPI_ID_BUTT }.
 * @param {ES_MIPI_CMD_S} *pMipiCmd HDMI command structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_MIPI_SetEnableCmd(ES_MIPI_ID_E mipiDev, const ES_MIPI_CMD_S *pMipiCmd);

/**
 * @description: This interface is used to set the disable command.
 * @param {ES_MIPI_ID_E} mipiDev MIPI Device ID. Value range{ ES_MIPI_ID_0 = 0, ES_MIPI_ID_BUTT }.
 * @param {ES_MIPI_CMD_S} *pMipiCmd HDMI command structure pointer.
 * @return ES_SUCCESS on success, VO error code otherwise.
 */
ES_S32 ES_MIPI_SetDisableCmd(ES_MIPI_ID_E mipiDev, const ES_MIPI_CMD_S *pMipiCmd);

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif /*__ES_VO_H__ */