/*
 * Copyright © 2023 ESWIN. All rights reserved.
 * Beijing ESWIN Computing Technology Co., Ltd and its affiliated companies ("ESWIN") retain
 * all intellectual property and proprietary rights in and to this software.Except as expressly
 * authorized by ESWIN, no part of the software may be released, copied, distributed,reproduced,
 * modified, adapted, translated, or created derivative work of, in whole or in part.
 * @Description: Define the API of VPS
 * @Author: ESWIN
 */

#ifndef __ES_VPS_H__
#define __ES_VPS_H__

#include "es_common.h"
#include "es_comm_video.h"
#include "es_comm_vps.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif
#endif /* __cplusplus */

/* Group Settings */

/**
 * Create a VPS Group. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param nId VPS group die id. Value range[ES_DIE_0, ES_DIE_1]
 * @param pGrpAttr VPS group attribute.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_CreateGrp(VPS_GRP vpsGrp, DIE_IDX nId, const VPS_GRP_ATTR_S *pGrpAttr);

/**
 * Destroy a VPS Group. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_DestroyGrp(VPS_GRP vpsGrp);

/**
 * Start a Group. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_StartGrp(VPS_GRP vpsGrp);

/**
 * Stop a Group. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_StopGrp(VPS_GRP vpsGrp);

/**
 * Reset a Group. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_ResetGrp(VPS_GRP vpsGrp);

/**
 * Get Group attribute. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param pGrpAttr VPS group attribute.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_GetGrpAttr(VPS_GRP vpsGrp, VPS_GRP_ATTR_S *pGrpAttr);

/**
 * Set Group attribute. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param pGrpAttr VPS group attribute.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_SetGrpAttr(VPS_GRP vpsGrp, const VPS_GRP_ATTR_S *pGrpAttr);

/**
 * Set Group crop. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param pCropInfo VPS crop info.
 * @param type  Appoint the hardware type to execute crop work. Only support HW_TYPE_HAE, HW_TYPE_3D,
 *              HW_TYPE_DWE, HW_TYPE_VSE.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note HW_TYPE_3D is not implemented now.
 *       When type is HW_TYPE_DWE, crop rect must be set: [roiX, roiY, image.width, image.height].
 *
 */
ES_S32 ES_VPS_SetGrpCrop(VPS_GRP vpsGrp, const VPS_CROP_INFO_S *pCropInfo, HWTYPE_E type);

/**
 * Get Group crop. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param pCropInfo VPS crop info.
 * @param pType Hardware type.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_GetGrpCrop(VPS_GRP vpsGrp, VPS_CROP_INFO_S *pCropInfo, HWTYPE_E *pType);

/**
 * Send a frame to group input buffer queue. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param pVideoFrame Video Frame.
 * @param milliSec Timeout value of waiting for function execution in milliseconds. -1 means block wait.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_SendFrame(VPS_GRP vpsGrp, const VIDEO_FRAME_INFO_S *pVideoFrame, ES_S32 milliSec);

/**
 * Set Group dewarp. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param pDewarp VPS dewarp parameter.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_SetGrpDewarp(VPS_GRP vpsGrp, const VPS_DEWARP_PARAMS_S *pDewarp);

/**
 * Get Group dewarp. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param pDewarp VPS dewarp parameter.
 *                  support mode:
 *                     VPS_DEWARP_MODEL_LENS_DISTORTION_CORRECTION,
 *                     VPS_DEWARP_MODEL_FISHEYE_EXPAND,
 *                     VPS_DEWARP_MODEL_SPLIT_SCREEN,
 *                     VPS_DEWARP_MODEL_FISHEYE_DEWARP,
 *                     VPS_DEWARP_MODEL_PERSPECTIVE,
 *                     VPS_DEWARP_MODEL_BYPASS,
 *                  crop:
 *                     [roiX, roiY, image.width, image.height]
 *                  resize:
 *                     scaleFactor; Up scale factor[1.0, 4.0]
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_GetGrpDewarp(VPS_GRP vpsGrp, VPS_DEWARP_PARAMS_S *pDewarp);

/**
 * Query group status. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param pQuery VPS group status. we can get:
 *               group received frame count
 *               chn processed frame count
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_QueryGrpStatus(VPS_GRP VPS_GRP, VPS_QUERY_STATUS_S *pQuery);

/**
 * Get multi out attribute. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param pAttr VPS Multi out attribute
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note Supports one in one/two/three out
 *
 */
ES_S32 ES_VPS_GetGrpMultiOutAttr(VPS_GRP vpsGrp, VPS_MULTI_OUT_ATTR_S *pAttr);

/**
 * Set multi out attribute. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param pAttr VPS Multi out attribute
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note If user want to use one group in multiple channels out feature, this function is mandatory.
 *       If only one group in one channel out, this function is optional.
 *       Once use this function, user need to make sure the channel enable status is same as the enable status
 *       in MultiOutAttr. Otherwise StartGrp will be failed.
 */
ES_S32 ES_VPS_SetGrpMultiOutAttr(VPS_GRP vpsGrp, const VPS_MULTI_OUT_ATTR_S *pAttr);

/* Chn Settings */

/**
 * Set chn attribute. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param pChnAttr Chn attribute
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_SetChnAttr(VPS_GRP vpsGrp, VPS_CHN vpsChn, const VPS_CHN_ATTR_S *pChnAttr);

/**
 * Get chn attribute. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param pChnAttr Chn attribute
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_GetChnAttr(VPS_GRP vpsGrp, VPS_CHN vpsChn, VPS_CHN_ATTR_S *pChnAttr);

/**
 * Enable chn. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_EnableChn(VPS_GRP vpsGrp, VPS_CHN vpsChn);

/**
 * Disable chn. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_DisableChn(VPS_GRP vpsGrp, VPS_CHN vpsChn);

/**
 * Set Chn crop attribute. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param pCropInfo VPS crop info.
 * @param type  Appoint the hardware type to execute crop work. Only support HW_TYPE_HAE, HW_TYPE_3D,
 *              HW_TYPE_DWE, HW_TYPE_VSE.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note HW_TYPE_3D is not implemented now.
 *
 */
ES_S32 ES_VPS_SetChnCrop(VPS_GRP vpsGrp, VPS_CHN vpsChn, const VPS_CROP_INFO_S *pCropInfo, HWTYPE_E type);

/**
 * Get Chn crop attribute. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param pCropInfo VPS crop info.
 * @param pType  Hardware type.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note HW_TYPE_3D is not implemented now.
 *       When type is HW_TYPE_DWE, crop rect must be set: [roiX, roiY, image.width, image.height].
 *
 */
ES_S32 ES_VPS_GetChnCrop(VPS_GRP vpsGrp, VPS_CHN vpsChn, VPS_CROP_INFO_S *pCropInfo, HWTYPE_E *pType);

/**
 * Set Chn Normalization attribute. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param pNormInfo VPS normalization info.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_SetChnNormalization(VPS_GRP vpsGrp, VPS_CHN vpsChn, const VPS_NORMALIZATION_INFO_S *pNormInfo);

/**
 * Get Chn Normalization attribute. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param pNormInfo VPS normalization info.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_GetChnNormalization(VPS_GRP vpsGrp, VPS_CHN vpsChn, VPS_NORMALIZATION_INFO_S *pNormInfo);

/**
 * Set the rotation of channel. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param rotation The rotation angle. Only support ROTATION_0, ROTATION_90, ROTATION_180, ROTATION_270.
 * @param type Appoint the hardware type to execute rotation work. Only support HW_TYPE_HAE, HW_TYPE_3D, HW_TYPE_DWE.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note HW_TYPE_3D is not implemented now.
 */
ES_S32 ES_VPS_SetChnRotation(VPS_GRP vpsGrp, VPS_CHN vpsChn, ROTATION_E rotation, HWTYPE_E type);

/**
 * Get the rotation of channel. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param pRotation The rotation angle. Only support ROTATION_0, ROTATION_90, ROTATION_180 and ROTATION_270.
 * @param pType the hardware type of execute rotation work. Only support HW_TYPE_HAE, HW_TYPE_3D, HW_TYPE_DWE.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note HW_TYPE_3D is not implemented now.
 */
ES_S32 ES_VPS_GetChnRotation(VPS_GRP vpsGrp, VPS_CHN vpsChn, ROTATION_E *pRotation, HWTYPE_E *pType);

/**
 * Get the frame of channel. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param pVideoFrame Video Frame.
 * @param milliSec Timeout value of waiting for function execution in milliseconds. -1 means block wait.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_GetChnFrame(VPS_GRP vpsGrp, VPS_CHN vpsChn, VIDEO_FRAME_INFO_S *pVideoFrame, ES_S32 milliSec);

/**
 * Release the frame of channel. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param pVideoFrame Video Frame.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_ReleaseChnFrame(VPS_GRP vpsGrp, VPS_CHN vpsChn, const VIDEO_FRAME_INFO_S *pVideoFrame);

/**
 * Channel attach a VB pool. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param vbPool VB Pool ID.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_AttachVbPool(VPS_GRP vpsGrp, VPS_CHN vpsChn, VB_POOL vbPool);

/**
 * Channel detach a VB pool. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_DetachVbPool(VPS_GRP vpsGrp, VPS_CHN vpsChn);

/* Module Param Settings */
ES_S32 ES_VPS_SetModParam(const VPS_MOD_PARAM_S *pModParam);
ES_S32 ES_VPS_GetModParam(VPS_MOD_PARAM_S *pModParam);

/**
 * Get channel fd. This interface is used in VPS link mode.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @return Chn fd >= 0.
 *
 */
ES_S32 ES_VPS_GetChnFd(VPS_GRP vpsGrp, VPS_CHN vpsChn);

/**
 * Create overlay handle.
 *
 * @return Valid pointer, ES_NULL when error.
 *
 * @note User need to ensure paired use ES_VPS_CreateOverlay and ES_VPS_DestroyOverlay to avoid memory leak.
 *
 */
VPS_OVERLAY_HANDLE ES_VPS_CreateOverlay();

/**
 * Destroy overlay handle.
 *
 * @param handle Overlay handle created by ES_VPS_CreateOverlay.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note User need to ensure paired use ES_VPS_CreateOverlay and ES_VPS_DestroyOverlay to avoid memory leak.
 *       Before destroy, make sure handle was detached from all the groups and channels, otherwise function
 *       will return errors. Handle will be destroied when nobody attach it.
 *
 */
ES_S32 ES_VPS_DestroyOverlay(const VPS_OVERLAY_HANDLE handle);

/**
 * Attach overlay handle to group.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param handle Overlay handle created by ES_VPS_CreateOverlay.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_AttachGrpOverlay(VPS_GRP vpsGrp, const VPS_OVERLAY_HANDLE handle);

/**
 * Detach overlay handle to group.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param handle Overlay handle created by ES_VPS_CreateOverlay.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_DetachGrpOverlay(VPS_GRP vpsGrp, const VPS_OVERLAY_HANDLE handle);

/**
 * Attach overlay handle to channel of group.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param handle Overlay handle created by ES_VPS_CreateOverlay.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_AttachChnOverlay(VPS_GRP vpsGrp, VPS_CHN vpsChn, const VPS_OVERLAY_HANDLE handle);

/**
 * Detach overlay handle to channel of group.
 *
 * @param vpsGrp VPS group ID, Value range[0, ES_VPS_MAX_GRP_NUM - 1].
 * @param vpsChn VPS chn ID. Value range[0, ES_VPS_MAX_CHN_NUM - 1].
 * @param handle Overlay handle created by ES_VPS_CreateOverlay.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_DetachChnOverlay(VPS_GRP vpsGrp, VPS_CHN vpsChn, const VPS_OVERLAY_HANDLE handle);

/**
 * Update the overlay group displays attribute for special overlay handle.
 *
 * @param handle Overlay handle created by ES_VPS_CreateOverlay.
 * @param pOverlays The overlay group displays attribute.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 */
ES_S32 ES_VPS_UpdateOverlay(const VPS_OVERLAY_HANDLE handle, const VPS_OVERLAY_GROUP_S *pOverlays);

/**
 * Init VPS. This interface is used in VPS link mode or single mode.
 *
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be init before using.
 *
 */
ES_S32 ES_VPS_Init();

/**
 * Deinit VPS. This interface is used in VPS link mode or single mode.
 *
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be deinit after using.
 *
 */
ES_S32 ES_VPS_Deinit();

/**
 * Crop and Resize. This interface is used in VPS single mode.
 *
 * @param pSrc  the source image.
 * @param pDst  the destination images, the count is defined by outputFlag
 * @param pSrcRect  src surface rect.
 * @param pDstRect  dst surface rect.
 * @param outputFlag  the output flags.
 *     If you want to use output0, flags can be 0x0 or 0x1;
 *     if you want to use output1, flags should be 0x2;
 *     if you want to use output2, flags should be 0x4;
 *     Both output1 and output2, flags should be 0x6
 * @param type  Appoint the hardware type to execute crop resize work. Only support HW_TYPE_HAE, HW_TYPE_3D,
 *              HW_TYPE_DWE, HW_TYPE_VSE.
 *
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be init before using.
 *       when type is HW_TYPE_DWE or HW_TYPE_VSE, pDstRect is not supported.
 *
 */
ES_S32 ES_VPS_CropResize(
    VIDEO_FRAME_S *pSrc, VIDEO_FRAME_S *pDst, RECT_S *pSrcRect, RECT_S *pDstRect, ES_S32 outputFlag, HWTYPE_E type);

/**
 * Rotation. This interface is used in VPS single mode.
 *
 * @param pSrc  the source image.
 * @param pDst  the destination image.
 * @param rot  Rotation angle: ROTATION_90, ROTATION_180, ROTATION_270.
 * @param type  Appoint the hardware type to execute rotation work. Only support HW_TYPE_HAE, HW_TYPE_3D, HW_TYPE_DWE.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be init before using.
 *
 */
ES_S32 ES_VPS_Rotation(VIDEO_FRAME_S *pSrc, VIDEO_FRAME_S *pDst, ROTATION_E rot, HWTYPE_E type);

/**
 * Line. This interface is used in VPS single mode.
 *
 * @param pDst  the destination image.
 * @param color  Set the line ARGB color.
 * @param pStart  line start point, it is a pointer array.
 * @param pEnd  line end point, it is a pointer array.
 * @param lineCount  line count.
 * @param type  Appoint the hardware type to execute draw line work. Only support HW_TYPE_HAE, HW_TYPE_3D,
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be init before using.
 *
 */
ES_S32 ES_VPS_Line(VIDEO_FRAME_S *pDst, ES_U32 color, POINT_S *pStart, POINT_S *pEnd, ES_S32 lineCount, HWTYPE_E type);

/**
 * Fill. This interface is used in VPS single mode.
 *
 * @param pDst  the destination image.
 * @param pColor  The ARGB color. It's a pointer array.
 * @param colorCnt  The count of color.
 * @param pRect  dst surface rect where will be filled. It is a pointer array.
 * @param rectCnt  dst surface rect count.
 * @param type  Appoint the hardware type to execute fill work. Only support HW_TYPE_HAE, HW_TYPE_3D,
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be init before using. If colorCnt < rectCnt, the remaining rect will use the last color.
 *
 */
ES_S32 ES_VPS_Fill(VIDEO_FRAME_S *pDst, ES_U32 *pColor, ES_S32 colorCnt, RECT_S *pRect, ES_S32 rectCnt, HWTYPE_E type);

/**
 * Do csc. This interface is used in VPS single mode.
 *
 * @param pSrc  the source image with source Format.
 * @param pDst  the destination image with target format.
 * @param type  Appoint the hardware type to execute csc work. Only support HW_TYPE_HAE, HW_TYPE_3D,
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be init before using.
 *
 */
ES_S32 ES_VPS_CSC(VIDEO_FRAME_S *pSrc, VIDEO_FRAME_S *pDst, HWTYPE_E type);

/**
 * Do alphaBlend. This interface is used in VPS single mode.
 *
 * @param pSrc  the source image with source Format.
 * @param pDst  the destination image with target format.
 * @param blendMod  set blend mode.
 * @param srcGlobalAlpha  set src surface global alpha value.
 * @param dstGlobalAlpha  set dst surface global alpha value.
 * @param type  Appoint the hardware type to execute alpha blend work. Only support HW_TYPE_HAE, HW_TYPE_3D,
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be init before using.
 *
 */
ES_S32 ES_VPS_AlphaBlending(VIDEO_FRAME_S *pSrc,
                            VIDEO_FRAME_S *pDst,
                            VPS_BLEND_MODE_E blendMod,
                            ES_S32 srcGlobalAlpha,
                            ES_S32 dstGlobalAlpha,
                            HWTYPE_E type);

/**
 * Do normalization, csc, cropResize. This interface is used in VPS single mode.
 *
 * @param pSrc  the source image with source Format.
 * @param pDst  the destination image with target format.
 * @param pParams  normalization parameter, support MIN_MAX, Z_SCORE mode.
 * @param pSrcRect  src surface rect.
 * @param pDstRect  dst surface rect.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be init before using.
 *
 */
ES_S32 ES_VPS_Normalization(
    VIDEO_FRAME_S *pSrc, VIDEO_FRAME_S *pDst, VPS_NORMALIZATION_PARAMS_S *pParams, RECT_S *pSrcRect, RECT_S *pDstRect);

/**
 * Do multi sources blit. This interface is used in VPS single mode.
 *
 * @param pSrcs  the source image with source Format.
 * @param srcCount  the source image count.
 * @param pSrcRects the rect to be selected of source image.
 * @param pRot  rotation angle: ROTATION_90, ROTATION_180, ROTATION_270. It is a pointer array.
 * @param pDstRects  the rect which source image locate at destination image.
 * @param pDst  the destination image with target format.
 * @param type  Appoint the hardware type to execute blit work. Only support HW_TYPE_HAE, HW_TYPE_3D.
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be init before using.
 *
 */
ES_S32 ES_VPS_MultiSourcesBlit(VIDEO_FRAME_S *pSrcs,
                               ES_S32 srcCount,
                               RECT_S *pSrcRects,
                               ROTATION_E *pRot,
                               RECT_S *pDstRects,
                               VIDEO_FRAME_S *pDst,
                               HWTYPE_E type);

/**
 * Do dewarp, crop , upscale. This interface is used in VPS single mode.
 *
 * @param pSrc  the source image with source Format.
 * @param pDst  the destination image with target format.
 * @param pParams  dewarp parameter.
 *                 support mode:
 *                     VPS_DEWARP_MODEL_LENS_DISTORTION_CORRECTION,
 *                     VPS_DEWARP_MODEL_FISHEYE_EXPAND,
 *                     VPS_DEWARP_MODEL_SPLIT_SCREEN,
 *                     VPS_DEWARP_MODEL_FISHEYE_DEWARP,
 *                     VPS_DEWARP_MODEL_PERSPECTIVE,
 *                     VPS_DEWARP_MODEL_BYPASS,
 *                  crop:
 *                     [roiX, roiY, image.width, image.height]
 *                  resize:
 *                     scaleFactor; Up scale factor[1.0, 4.0]
 *
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be init before using.
 *
 */
ES_S32 ES_VPS_Dewarp(VIDEO_FRAME_S *pSrc, VIDEO_FRAME_S *pDst, const VPS_DEWARP_PARAMS_S *pParams);

/**
 * Set VPS property for single mode. This interface is used in VPS single mode.
 *
 * @param pVpsProperty     VPS property, based on pVpsProperty->type for really property
 *
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be init before using.
 * Those values bind with thread. For different thread, you need set those values again
 *
 */
ES_S32 ES_VPS_SetProperty(const VPS_PROPERTY_S *pVpsProperty);

/**
 * Get VPS property for single mode. This interface is used in VPS single mode.
 *
 * @param pVpsProperty     VPS property, pVpsProperty->type should be set
 *
 * @return ES_SUCCESS on success, VPS error code otherwise
 *
 * @note VPS must be init before using.
 * Those values bind with thread. For different thread, you may get different value
 *
 */
ES_S32 ES_VPS_GetProperty(VPS_PROPERTY_S *pVpsProperty);

#ifdef __cplusplus
#if __cplusplus
}
#endif
#endif /* __cplusplus */

#endif /* __ES_VPS_H__ */
